@@ -1,19 +1,18 @@
 """ S3-backed pypi server """
-from pyramid.config import Configurator
-from pyramid.renderers import JSON
 import datetime
-from pyramid.path import DottedNameResolver
-from pyramid.renderers import render
-from pyramid.settings import asbool
-from pyramid_beaker import session_factory_from_settings
 
+import logging
+from pyramid.config import Configurator
+from pyramid.renderers import JSON, render
+from pyramid.settings import asbool, aslist
+from pyramid_beaker import session_factory_from_settings
+from six.moves.urllib.parse import urlencode  # pylint: disable=F0401,E0611
+
+from .cache import get_cache_impl
 from .route import Root
 
-
-try:
-    from ._version import *  # pylint: disable=F0401,W0401
-except ImportError:  # pragma: no cover
-    __version__ = 'unknown'
+__version__ = '0.2.0'
+LOG = logging.getLogger(__name__)
 
 
 def to_json(value):
@@ -25,11 +24,13 @@ json_renderer.add_adapter(datetime.datetime, lambda obj, r:
                           float(obj.strftime('%s.%f')))
 
 
-def _app_url(request, *paths):
+def _app_url(request, *paths, **params):
     """ Get the base url for the root of the app plus an optional path """
     path = '/'.join(paths)
     if not path.startswith('/'):
         path = '/' + path
+    if params:
+        path += '?' + urlencode(params)
     return request.application_url + path
 
 
@@ -59,24 +60,25 @@ def includeme(config):
     config.set_session_factory(session_factory_from_settings(settings))
 
     # PYPICLOUD SETTINGS
+    default_url = 'http://pypi.python.org/simple'
     config.registry.fallback_url = settings.get('pypi.fallback_url',
-                                                'http://pypi.python.org/simple')
-    config.registry.use_fallback = asbool(settings.get('pypi.use_fallback',
-                                                       True))
-    realm = settings.get('pypi.realm', 'pypi')
-    config.registry.realm = realm
+                                                default_url)
+
+    fallback_modes = aslist(settings.get('pypi.fallback', ['redirect']))
+    # Compatibility with the deprecated pypi.use_fallback option
+    if 'pypi.fallback' not in settings and 'pypi.use_fallback' in settings:
+        LOG.warn("Using deprecated option 'pypi.use_fallback'")
+        use_fallback = asbool(settings['pypi.use_fallback'])
+        fallback_modes = ['redirect'] if use_fallback else []
+    modes = ('redirect', 'cache', 'none')
+    for mode in fallback_modes:
+        if mode not in modes:
+            raise ValueError("Invalid value for 'pypi.fallback'. "
+                             "Must be one of %s" % ', '.join(modes))
+    config.registry.fallback = fallback_modes
 
     # CACHING DATABASE SETTINGS
-    resolver = DottedNameResolver(__name__)
-    dotted_cache = settings.get('pypi.db', 'sql')
-    if dotted_cache == 'sql':
-        dotted_cache = 'pypicloud.cache.SQLCache'
-    elif dotted_cache == 'redis':
-        dotted_cache = 'pypicloud.cache.RedisCache'
-    cache_impl = resolver.resolve(dotted_cache)
-
-    cache_impl.configure(config)
-    cache_impl.reload_if_needed()
+    cache_impl = get_cache_impl(settings)
 
     config.add_request_method(cache_impl, name='db', reify=True)
 
@@ -96,5 +98,5 @@ def main(config, **settings):
     """
     config = Configurator(settings=settings)
     config.include('pypicloud')
-    config.scan(ignore='pypicloud.tests')
+    config.scan('pypicloud.views')
     return config.make_wsgi_app()
